Imports C1.C1Pdf
Imports C1.Win.C1Document
Imports C1.Win.C1Document.Util
Imports C1.Win.FlexReport
Imports System.Collections.Generic
Imports System.Drawing
Imports System.Globalization
Imports System.Runtime.CompilerServices

Module Extension
    Sub New()
    End Sub

    Private pdfDocument As C1PdfDocument = Nothing
    Private _pdfGraphics As C1PdfGraphics = Nothing
    Private c1FlexReports As New List(Of C1FlexReport)()
    Private page As Integer = 1
    Private totalPages As Integer = 0

    <System.Runtime.CompilerServices.Extension()>
    Public Sub AddC1FlexReport(_pdfDoc As C1PdfDocument, c1FlexReport As C1FlexReport)
        pdfDocument = _pdfDoc
        'Create C1PdfGraphics
        _pdfGraphics = New C1PdfGraphics(_pdfDoc)
        Dim _flex As New C1FlexReport()
        'Copy report definition
        _flex.ReportDefinition = c1FlexReport.ReportDefinition
        'Add report to list
        c1FlexReports.Add(_flex)
    End Sub

    <System.Runtime.CompilerServices.Extension()>
    Public Sub RenderPdf(_pdfDoc As C1PdfDocument, Optional IsPageSequential As Boolean = False)
        'Render report(s) and calculate total page count for all report(s)
        For Each c1FlexReport As C1FlexReport In c1FlexReports
            c1FlexReport.Render()
            totalPages += c1FlexReport.PageCount
        Next
        'For sequential page number
        If IsPageSequential Then
            For Each c1FlexReport As C1FlexReport In c1FlexReports
                AddHandler c1FlexReport.StartPage, AddressOf C1FlexReport_StartPage
                c1FlexReport.Render()
            Next
        End If

        For Each c1FlexReport As C1FlexReport In c1FlexReports
            Dim _doc As C1Document = c1FlexReport.Document
            For i As Integer = 0 To c1FlexReport.PageCount - 1
                If _pdfDoc.Pages.Count > 1 OrElse i > 0 OrElse c1FlexReports.IndexOf(c1FlexReport) > 0 Then
                    'New page added in C1PdfDocument
                    _pdfDoc.NewPage()
                End If

                'Get C1Page from C1FlexReprot using index
                Dim page As C1Page = c1FlexReport.GetPage(i)
                _pdfDoc.PageSize = New SizeF(CSng(Unit.Convert(page.FullWidth, _doc.MeasurementUnits, UnitTypeEnum.Point)), CSng(Unit.Convert(page.FullHeight, _doc.MeasurementUnits, UnitTypeEnum.Point)))
                ' calculate scale factor
                Dim sx As Double = _pdfDoc.PageSize.Width / Unit.Convert(page.FullWidth, _doc.MeasurementUnits, UnitTypeEnum.Dip)
                Dim sy As Double = _pdfDoc.PageSize.Height / Unit.Convert(page.FullHeight, _doc.MeasurementUnits, UnitTypeEnum.Dip)

                ' translate/scale
                _pdfDoc.Write("{0} 0 0 {1} {2} {3} cm" & vbCr & vbLf, DoubleToString(sx), DoubleToString(sy), DoubleToString(0), DoubleToString(-0 + _pdfDoc.PageRectangle.Height * (1 - sy)))

                ' draw page using pdf graphics
                page.Draw(_pdfGraphics, New Point(0, 0), Nothing, Nothing, DrawAttrs.[Default])
            Next
            AddOutlines(_doc.Outlines)
        Next
        c1FlexReports = New List(Of C1FlexReport)()
    End Sub

    'Convert Double to String
    Private Function DoubleToString(_double As Double) As String
        Return _double.ToString("0.###", CultureInfo.InvariantCulture)
    End Function

    Private Sub AddOutlines(onc As OutlineNodeCollection)
        For Each outlineNode As OutlineNode In onc
            If TypeOf outlineNode.LinkTarget Is C1LinkTarget Then
                Dim target = DirectCast(outlineNode.LinkTarget, C1LinkTarget).Bookmark
                pdfDocument.AddBookmark(outlineNode.Caption, outlineNode.Level, target)
                If outlineNode.HasChildren Then
                    AddOutlines(outlineNode.Children)
                End If
            End If
        Next
    End Sub

    Private Sub C1FlexReport_StartPage(sender As Object, e As ReportEventArgs)
        'Update the text of page footer for printing the page number and page count in sequence
        TryCast(TryCast(sender, C1FlexReport).Fields("ftrRight"), TextField).Text = "Page " & page & " of " & totalPages
        page += 1
    End Sub
End Module